@@ -30,16 +30,16 @@ from rejected import data
 LOGGER = logging.getLogger(__name__)
 
 
-def import_consumer(consumer):
+def import_consumer(value):
     """Pass in a string in the format of foo.Bar, foo.bar.Baz, foo.bar.baz.Qux
     and it will return a handle to the class
 
-    :param str consumer: The consumer class in module.Consumer format
+    :param str value: The consumer class in module.Consumer format
     :return: tuple(Class, str)
 
     """
 
-    parts = consumer.split('.')
+    parts = value.split('.')
     import_name = '.'.join(parts[0:-1])
     import_handle = importlib.import_module(import_name)
     if hasattr(import_handle, '__version__'):
@@ -57,7 +57,6 @@ class Process(multiprocessing.Process, common.State):
 
     """
     _AMQP_APP_ID = 'rejected/%s' % __version__
-    _QOS_PREFETCH_COUNT = 1
 
     # Additional State constants
     STATE_PROCESSING = 0x04
@@ -76,11 +75,7 @@ class Process(multiprocessing.Process, common.State):
     TIME_WAITED = 'idle_time'
     UNHANDLED_EXCEPTIONS = 'unhandled_exceptions'
 
-    _HBINTERVAL = 120
-
-    # Locations to search for newrelic ini files
-    INI_DIRS = ['.', '/etc/', '/etc/newrelic']
-    INI_FILE = 'newrelic.ini'
+    _HBINTERVAL = 300
 
     # Default message pre-allocation value
     _QOS_PREFETCH_COUNT = 1
@@ -91,6 +86,8 @@ class Process(multiprocessing.Process, common.State):
     _MAX_SHUTDOWN_WAIT = 5
     _RECONNECT_DELAY = 10
 
+    _STATSD_FORMAT = 'rejected.consumer.{0}.{1}:{2}|c\n'
+
     def __init__(self, group=None, target=None, name=None, args=(),
                  kwargs=None):
         if kwargs is None:
@@ -100,20 +97,24 @@ class Process(multiprocessing.Process, common.State):
         self._application = None
         self._channel = None
         self._config = None
+        self._connection = None
         self._connection_id = 0
         self._connection_name = None
         self._connections = None
         self._consumer = None
+        self._consumer_name = None
         self._counts = self.new_counter_dict()
         self._dynamic_qos = True
-        self._hb_interval = self._HBINTERVAL
+        self._ioloop = None
         self._last_counts = dict()
         self._last_failure = 0
         self._last_stats_time = None
         self._logging_config = dict()
         self._message_connection_id = None
+        self._max_error_count = self._MAX_ERROR_COUNT
         self._max_frame_size = pika.spec.FRAME_MAX_SIZE
         self._qos_prefetch = None
+        self._queue_name = None
         self._prepend_path = None
         self._state = self.STATE_INITIALIZING
         self._state_start = time.time()
@@ -121,6 +122,9 @@ class Process(multiprocessing.Process, common.State):
         self._statsd = False
         self._statsd_host = 'localhost'
         self._statsd_port = 8125
+        self._statsd_socket = socket.socket(socket.AF_INET,
+                                            socket.SOCK_DGRAM,
+                                            socket.IPPROTO_UDP)
 
         # Override ACTIVE with PROCESSING
         self._STATES[0x04] = 'Processing'
@@ -136,7 +140,7 @@ class Process(multiprocessing.Process, common.State):
             self.increment_count(self.CLOSED_ON_COMPLETE)
             return
         LOGGER.debug('Acking %s', delivery_tag)
-        self._channel.basic_ack(delivery_tag=delivery_tag, multiple=True)
+        self._channel.basic_ack(delivery_tag=delivery_tag)
         self.increment_count(self.ACKED)
 
     def add_on_channel_close_callback(self):
@@ -171,6 +175,9 @@ class Process(multiprocessing.Process, common.State):
         :rtype: bool or int
 
         """
+        if not self._last_stats_time:
+            return
+
         qos_prefetch = self.dynamic_qos_pretch
 
         # Don't change anything
@@ -187,6 +194,7 @@ class Process(multiprocessing.Process, common.State):
 
         # If calculated QoS exceeds max
         if qos_prefetch > self._QOS_MAX:
+            LOGGER.debug('Hit QoS Max ceiling of %i', self._QOS_MAX)
             return self.set_qos_prefetch(self._QOS_MAX)
 
         # Set to base value if QoS calc is < than the base
@@ -230,27 +238,29 @@ class Process(multiprocessing.Process, common.State):
         LOGGER.info('Closing connection')
         self._connection.close()
 
-    def connect_to_rabbitmq(self, config, name):
+    def connect_to_rabbitmq(self, cfg, name):
         """Connect to RabbitMQ returning the connection handle.
 
-        :param dict config: The Connections section of the configuration
+        :param dict cfg: The Connections section of the configuration
         :param str name: The name of the connection
         :rtype: pika.adapters.tornado_connection.TornadoConnection
 
         """
         LOGGER.debug('Connecting to %s:%i:%s as %s',
-                     config[name]['host'], config[name]['port'],
-                     config[name]['vhost'], config[name]['user'])
+                     cfg[name]['host'], cfg[name]['port'],
+                     cfg[name]['vhost'], cfg[name]['user'])
         self.set_state(self.STATE_CONNECTING)
         self._connection_id += 1
-        parameters = self.get_connection_parameters(config[name]['host'],
-                                                    config[name]['port'],
-                                                    config[name]['vhost'],
-                                                    config[name]['user'],
-                                                    config[name]['pass'])
+        hb_interval = cfg[name].get('heartbeat_interval', self._HBINTERVAL)
+        parameters = self.get_connection_parameters(cfg[name]['host'],
+                                                    cfg[name]['port'],
+                                                    cfg[name]['vhost'],
+                                                    cfg[name]['user'],
+                                                    cfg[name]['pass'],
+                                                    hb_interval)
         return tornado_connection.TornadoConnection(parameters,
                                                     self.on_connection_open,
-                                                    stop_ioloop_on_close=True)
+                                                    stop_ioloop_on_close=False)
 
     def count(self, stat):
         """Return the current count quantity for a specific stat.
@@ -287,22 +297,24 @@ class Process(multiprocessing.Process, common.State):
         LOGGER.debug('Calculated prefetch value: %i', value)
         return value
 
-    def get_config(self, config, number, name, connection):
+    @staticmethod
+    def get_config(cfg, number, name, connection):
         """Initialize a new consumer thread, setting defaults and config values
 
-        :param dict config: Consumer config section from YAML File
+        :param dict cfg: Consumer config section from YAML File
         :param int number: The identification number for the consumer
         :param str name: The name of the consumer
         :param str connection: The name of the connection):
         :rtype: dict
 
         """
-        return {'connection': config['Connections'][connection],
+        return {'connection': cfg['Connections'][connection],
                 'connection_name': connection,
                 'consumer_name': name,
                 'process_name': '%s_%i_tag_%i' % (name, os.getpid(), number)}
 
-    def get_connection_parameters(self, host, port, vhost, username, password):
+    def get_connection_parameters(self, host, port, vhost, username, password,
+                                  heartbeat_interval):
         """Return connection parameters for a pika connection.
 
         :param str host: The RabbitMQ host to connect to
@@ -310,6 +322,7 @@ class Process(multiprocessing.Process, common.State):
         :param str vhost: The virtual host
         :param str username: The username to use
         :param str password: The password to use
+        :param int heartbeat_interval: AMQP Heartbeat interval
         :rtype: pika.ConnectionParameters
 
         """
@@ -317,21 +330,22 @@ class Process(multiprocessing.Process, common.State):
         return pika.ConnectionParameters(host, port, vhost, credentials,
                                          frame_max=self._max_frame_size,
                                          socket_timeout=10,
-                                         heartbeat_interval=self._hb_interval)
+                                         heartbeat_interval=heartbeat_interval)
 
-    def get_consumer(self, config):
+    @staticmethod
+    def get_consumer(cfg):
         """Import and create a new instance of the configured message consumer.
 
-        :param dict config: The named consumer section of the configuration
+        :param dict cfg: The named consumer section of the configuration
         :rtype: instance
         :raises: ImportError
 
         """
         try:
-            consumer_, version = import_consumer(config['consumer'])
+            consumer_, version = import_consumer(cfg['consumer'])
         except ImportError as error:
             LOGGER.exception('Error importing the consumer %s: %s',
-                             config['consumer'], error)
+                             cfg['consumer'], error)
             exc_type, exc_value, exc_traceback = sys.exc_info()
             lines = traceback.extract_tb(exc_traceback)
             for line in lines:
@@ -339,27 +353,21 @@ class Process(multiprocessing.Process, common.State):
             return
 
         if version:
-            LOGGER.info('Creating consumer %s v%s', config['consumer'], version)
+            LOGGER.info('Creating consumer %s v%s', cfg['consumer'], version)
         else:
-            LOGGER.info('Creating consumer %s', config['consumer'])
+            LOGGER.info('Creating consumer %s', cfg['consumer'])
 
         kwargs = {}
-        if 'config' in config:
-            kwargs['configuration'] = config.get('config', dict())
-
-        try:
-            if consumer_.WANTS_CONNECTION_CONFIG:
-                kwargs['connections'] = self._connections
-        except AttributeError:
-            pass
+        if 'config' in cfg:
+            kwargs['configuration'] = cfg.get('config', dict())
 
         try:
             return consumer_(**kwargs)
         except TypeError:
-            return consumer_(config.get('config', dict()))
+            return consumer_(cfg.get('config', dict()))
         except Exception as error:
             LOGGER.error('Error creating the consumer "%s": %s',
-                         config['consumer'], error)
+                         cfg['consumer'], error)
             exc_type, exc_value, exc_traceback = sys.exc_info()
             lines = traceback.extract_tb(exc_traceback)
             for line in lines:
@@ -385,9 +393,11 @@ class Process(multiprocessing.Process, common.State):
 
         """
         self.start_message_processing()
+
         # Try and process the message
         try:
-            self._consumer.process(message)
+            LOGGER.debug('Processing message')
+            self._consumer.receive(message)
 
         except KeyboardInterrupt:
 
@@ -557,7 +567,7 @@ class Process(multiprocessing.Process, common.State):
 
         # Stop the IOLoop
         LOGGER.debug('Stopping IOLoop')
-        self._connection.ioloop.stop()
+        self._ioloop.stop()
 
         # Note that shutdown is complete and set the state accordingly
         self.set_state(self.STATE_STOPPED)
@@ -573,15 +583,20 @@ class Process(multiprocessing.Process, common.State):
 
         """
         values = dict()
+        if self.is_processing or self.is_idle:
+            self.calculate_qos_prefetch()
         for key in self._counts.keys():
             values[key] = self._counts[key] - self._last_counts.get(key, 0)
             self._last_counts[key] = self._counts[key]
             if self._statsd:
                 self.send_counter_to_statsd(key, values[key])
+
         self._stats_queue.put({'name': self.name,
                                'consumer_name': self._consumer_name,
-                               'counts': values})
+                               'counts': values}, True)
+
         LOGGER.debug('Currently %s: %r', self.state_description, values)
+        self._last_stats_time = time.time()
         signal.siginterrupt(signal.SIGPROF, False)
 
     def open_channel(self):
@@ -589,12 +604,12 @@ class Process(multiprocessing.Process, common.State):
         LOGGER.debug('Opening a channel on %r', self._connection)
         self._connection.channel(self.on_channel_open)
 
-    def process(self, channel=None, method=None, header=None, body=None):
+    def process(self, channel=None, method=None, properties=None, body=None):
         """Process a message from Rabbit
 
         :param pika.channel.Channel channel: The channel the message was sent on
         :param pika.frames.MethodFrame method: The method frame
-        :param pika.frames.HeaderFrame header: The header frame
+        :param pika.spec.BasicProperties properties: The message properties
         :param str body: The message body
 
         """
@@ -604,13 +619,14 @@ class Process(multiprocessing.Process, common.State):
             return self.reject(method.delivery_tag, True)
         self.set_state(self.STATE_PROCESSING)
         LOGGER.debug('Received message #%s', method.delivery_tag)
-        message = data.Message(channel, method, header, body)
+        message = data.Message(channel, method, properties, body)
         if method.redelivered:
             self.increment_count(self.REDELIVERED)
         self._state_start = time.time()
         if not self.invoke_consumer(message):
             LOGGER.debug('Bypassing ack due to False return from _process')
             return
+
         self.increment_count(self.PROCESSED)
         if self._ack:
             self.ack_message(method.delivery_tag)
@@ -658,15 +674,19 @@ class Process(multiprocessing.Process, common.State):
         if self._connection:
             if self._connection.socket:
                 fd = self._connection.socket.fileno()
-                self._connection.ioloop.remove_handler(fd)
-            self._connection.ioloop.stop()
-            del self._connection
-        else:
-            _ioloop = ioloop.IOLoop.instance()
-            _ioloop.stop()
-        time.sleep(self._RECONNECT_DELAY)
+                self._ioloop.remove_handler(fd)
+            self._connection = None
+
+        self._ioloop.add_timeout(time.time() + self._RECONNECT_DELAY,
+                                 self._reconnect)
+
+    def _reconnect(self):
+        """Create and set the RabbitMQ connection"""
+        LOGGER.info('Connecting to RabbitMQ')
+        self.reset_failure_counter()
         self._connection = self.connect_to_rabbitmq(self._connections,
                                                     self._connection_name)
+        self.setup_signal_handlers()
 
     def record_exception(self, error, handled=False, exc_info=None):
         """Record an exception
@@ -677,13 +697,14 @@ class Process(multiprocessing.Process, common.State):
         """
         self.increment_count(self.ERROR)
         if handled:
-            LOGGER.exception('Processor handled %s: %s',
-                             error.__class__.__name__, error)
+            if not isinstance(error, consumer.MessageException):
+                LOGGER.exception('Processor handled %s: %s',
+                                 error.__class__.__name__, error)
         else:
             LOGGER.exception('Processor threw an uncaught exception %s: %s',
                              error.__class__.__name__, error)
             self.increment_count(self.UNHANDLED_EXCEPTIONS)
-        if exc_info:
+        if not isinstance(error, consumer.MessageException) and exc_info:
             formatted_lines = traceback.format_exception(*exc_info)
             for offset, line in enumerate(formatted_lines):
                 LOGGER.debug('(%s) %i: %s', error.__class__.__name__,
@@ -745,6 +766,7 @@ class Process(multiprocessing.Process, common.State):
 
     def _run(self):
         """Run method that can be profiled"""
+        self._ioloop = ioloop.IOLoop.instance()
         common.add_null_handler()
         try:
             self.setup(self._kwargs['config'],
@@ -754,14 +776,14 @@ class Process(multiprocessing.Process, common.State):
                        self._kwargs['logging_config'])
         except ImportError as error:
             name = self._kwargs['consumer_name']
-            consumer = self._kwargs['config']['Consumers'][name]['consumer']
+            classname = self._kwargs['config']['Consumers'][name]['consumer']
             LOGGER.critical('Could not import %s, stopping process: %r',
-                            consumer, error)
+                            classname, error)
             return
 
         if not self.is_stopped:
             try:
-                self._connection.ioloop.start()
+                self._ioloop.start()
             except KeyboardInterrupt:
                 LOGGER.warning('CTRL-C while waiting for clean shutdown')
 
@@ -772,12 +794,11 @@ class Process(multiprocessing.Process, common.State):
         :param int|float value: The count
 
         """
-        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        sock.connect((self._statsd_host, self._statsd_port))
-        key = '.'.join(['rejected', 'consumers', self._kwargs['consumer_name'],
-                        counter])
-        sock.send('%s:%i|c\n' % (key, math.ceil(value)))
-        sock.close()
+        payload = self._STATSD_FORMAT.format(self._consumer_name,
+                                             counter,
+                                             math.ceil(value))
+        self._statsd_socket.sendto(payload, (self._statsd_host,
+                                             self._statsd_port))
 
     def set_qos_prefetch(self, value=None):
         """Set the QOS Prefetch count for the channel.
@@ -808,15 +829,15 @@ class Process(multiprocessing.Process, common.State):
         # Use the parent object to set the state
         super(Process, self).set_state(new_state)
 
-    def setup(self, config, connection_name, consumer_name, stats_queue,
+    def setup(self, cfg, connection_name, consumer_name, stats_queue,
               logging_config):
         """Initialize the consumer, setting up needed attributes and connecting
         to RabbitMQ.
 
-        :param dict config: Consumer config section
-\        :param str connection_name: The name of the connection
+        :param dict cfg: Consumer config section
+        :param str connection_name: The name of the connection
         :param str consumer_name: Consumer name for config
-        :param multiprocessing.SimpleQueue stats_queue: Queue to MCP
+        :param multiprocessing.Queue stats_queue: Queue to MCP
         :param dict logging_config: Logging config from YAML file
 
         """
@@ -824,31 +845,31 @@ class Process(multiprocessing.Process, common.State):
                     self.name, connection_name)
         self._logging_config = logging_config
         self._stats_queue = stats_queue
+        LOGGER.info('Stats queue: %r', self._stats_queue)
         self._connection_name = connection_name
         self._consumer_name = consumer_name
-        self._config = config['Consumers'][consumer_name]
-        self._connections = config['Connections']
+        self._config = cfg['Consumers'][consumer_name]
+        self._connections = cfg['Connections']
         self._consumer = self.get_consumer(self._config)
 
         if not self._consumer:
             LOGGER.critical('Could not import and start processor')
             self.set_state(self.STATE_STOPPED)
-            os._exit(1)
+            exit(1)
 
         self._queue_name = self._config['queue']
-        self._dynamic_qos = self._config.get('dynamic_qos', False)
         self._ack = self._config.get('ack', True)
+        self._dynamic_qos = self._config.get('dynamic_qos', False)
         self._max_error_count = int(self._config.get('max_errors',
                                                      self._MAX_ERROR_COUNT))
-        self._hb_interval = self._config.get('heartbeat_interval',
-                                             self._HBINTERVAL)
         self._max_frame_size = self._config.get('max_frame_size',
                                                 pika.spec.FRAME_MAX_SIZE)
+
         self._statsd = False
-        if 'statsd' in config and config['statsd'].get('enabled', False):
+        if 'statsd' in cfg and cfg['statsd'].get('enabled', False):
             self._statsd = True
-            self._statsd_host = config['statsd'].get('host', 'localhost')
-            self._statsd_port = config['statsd'].get('port', 8125)
+            self._statsd_host = cfg['statsd'].get('host', 'localhost')
+            self._statsd_port = cfg['statsd'].get('port', 8125)
 
         self.reset_failure_counter()
         self.setup_signal_handlers()
@@ -887,6 +908,7 @@ class Process(multiprocessing.Process, common.State):
 
         signal.siginterrupt(signal.SIGPROF, False)
         signal.siginterrupt(signal.SIGABRT, False)
+        LOGGER.debug('Signal handlers setup')
 
     def start_message_processing(self):
         """Keep track of the connection in case RabbitMQ disconnects while the
